from __future__ import print_function
from pylibcontainer import image
import click

@click.group()
def pylibcontainer():
    pass

pylibcontainer.add_command(image.run)

if __name__ == "__main__":
    pylibcontainer()



from __future__ import print_function

import tarfile
from os.path import join, exists, basename
from os import makedirs
from tempfile import gettempdir, mkdtemp
from shutil import move

def extract_layer(layer_fn):
    cache_key = basename(layer_fn)
    cache_prefix = join(gettempdir(), "ctinspector", "layer.cache")
    cache_path = join(cache_prefix, cache_key)
    if not exists(cache_prefix):
        makedirs(cache_prefix, 0o700)

    # Image is not yet cached, save and extract it
    if not exists(cache_path):
        tarfile.os.mknod = lambda x, y, z: 0  # Monkey patch mknod because some layers include devices
        with tarfile.open(layer_fn) as image_tar:
            tmp_dir = mkdtemp()
            image_tar.extractall(tmp_dir)
        move(tmp_dir, cache_path)
    else:
        print("Using rootfs dir from tmp cache")

    return cache_path



from __future__ import print_function
import os
import shutil
import requests
import click
from tempfile import NamedTemporaryFile
from hashlib import sha256
from os.path import expanduser, join, exists, basename
from pylibcontainer.utils import eprint, HumanSize
from pylibcontainer.tar import extract_layer
from pylibcontainer import container
from clint.textui import progress


CACHE_PATH = join(expanduser("~"), ".pylibcontainer", "images_cache")


class Cache(object):
    cache_dir = CACHE_PATH

    """ Provides an image cahcing mechanism on disk """
    def __init__(self):
        if not exists(CACHE_PATH):
            os.makedirs(CACHE_PATH, 0o700)

    def get(self, cache_key, default=None):
        """ return info for cached file """
        cache_hash = sha256(cache_key).hexdigest()
        cache_fn = join(CACHE_PATH, "url_" + cache_hash)

        if exists(cache_fn):
            file_stat = os.stat(cache_fn)
            last_modified = file_stat.st_mtime
            file_size = file_stat.st_size
            return cache_fn, cache_hash, last_modified, file_size

        return default

    def put(self, filename, cache_key):
        """ put a file into cache """
        cache_hash = sha256(cache_key).hexdigest()
        cache_fn = join(CACHE_PATH, "url_" + cache_hash)
        shutil.move(filename, cache_fn)
        return cache_hash, cache_fn

def download(image_url):
    """ Download image (if not found in cache) and return it's filename """
    response = requests.head(image_url)
    if response.status_code != 200:
        eprint("http error code", response.status_code)
        exit(2)
    cache = Cache()
    cached_image = cache.get(image_url)
    if cached_image:
        cache_fn, cache_hash, last_modified, file_size = cached_image
        remote_file_size = int(response.headers.get('Content-Length'))
        remote_last_modified = response.headers.get('Last-Modified')
        if remote_file_size == file_size and remote_last_modified > last_modified:
            print("Using file from cache")
            return cache_hash, cache_fn
    file_size = int(response.headers.get('Content-Length', 0))
    print("Downloading image... {0} [{1:.2S}]".format(basename(image_url), HumanSize(file_size)))
    response = requests.get(image_url, stream=True)
    with NamedTemporaryFile(delete=False) as tmp_file:
        for chunk in progress.bar(response.iter_content(chunk_size=1024), expected_size=(file_size/1024) + 1):
            if chunk:
                tmp_file.write(chunk)
                tmp_file.flush()
    return cache.put(tmp_file.name, image_url)


@click.command()
@click.argument('image_url')
@click.argument('command', nargs=-1, type=click.Path())
def run(image_url, command):
    image_protocol = image_url.split(':')[0].lower()
    if image_protocol in ['http', 'https']:
        _, image_fn = download(image_url)
    else:
        _, image_fn = sha256(image_url).hexdigest(), image_url
    rootfs = extract_layer(image_fn)
    print("Executing %s" % ' '.join(command))
    container.runc(rootfs, command)




from __future__ import print_function
import os
from tmsyscall.unshare import unshare, CLONE_NEWNS, CLONE_NEWUTS, CLONE_NEWIPC, CLONE_NEWPID, CLONE_NEWNET
from tmsyscall.mount import mount, unmount, mount_procfs, MS_BIND, MS_PRIVATE, MS_REC, MNT_DETACH
from tmsyscall.pivot_root import pivot_root
from os.path import exists, join


def setup_process_isolation(rootfs_path):
    # Detach from parent's mount, hostname, ipc and net  namespaces
    unshare(CLONE_NEWNS| CLONE_NEWUTS | CLONE_NEWIPC| CLONE_NEWNET)

    # Set mount propagation to private recursively. Hopefully equivalent to
    #    mount --make-rprivate /
    # This is needed to prevent mounts in this container leaking to the parent.
    mount('none', '/', None, MS_REC|MS_PRIVATE, "")

    root_fs = rootfs_path

    # This bind mount call is needed to satisfy a requirement of the `pivotroot` system call
    #   "new_root and put_old must not be on the same file system as the current root"
    # It is achieved by mounting "new_root" as a bind mount to "new root"
    mount(root_fs, root_fs, "", MS_BIND|MS_REC, "")

    old_root = join(root_fs, ".old_root")
    if not exists(old_root):
        os.makedirs(old_root, 0o700)
    pivot_root(root_fs, old_root)

    # We don't want the host root to be available to the container
    unmount("/.old_root", MNT_DETACH)
    os.rmdir("/.old_root")

    os.chdir("/")

    # Mount /proc for apps that need it
    if not exists("proc"):
        os.makedirs("proc", 0o700)
    mount_procfs('.')  # py


def child(rootfs_path, cmd):
    setup_process_isolation(rootfs_path)
    os.execvp(cmd[0], cmd)

def parent(child_pid):
    pid, status = os.waitpid(child_pid, 0)
    print("wait returned, pid = %d, status = %d" % (pid, status))


def runc(rootfs_path, command):
    # Detach from pid namespace so that our child get's a clean /proc with the new namespace
    unshare(CLONE_NEWPID)
    pid = os.fork()
    if pid == 0:
        child(rootfs_path, command)
    else:
        parent(pid)



from __future__ import print_function
import sys
import math

def eprint(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)

class HumanSize(int):
    """ define a size class to allow custom formatting
        Implements a format specifier of S for the size class - which displays a human readable in b, kb, Mb etc
    """
    def __format__(self, fmt):
        if fmt == "" or fmt[-1] != "S":
            if fmt[-1].tolower() in ['b', 'c', 'd', 'o', 'x', 'n', 'e', 'f', 'g', '%']:
                # Numeric format.
                return int(self).__format__(fmt)
            return str(self).__format__(fmt)

        val, text = float(self), ["b ", "Kb", "Mb", "Gb", "Tb", "Pb"]
        if val < 1:
            # Can't take log(0) in any base.
            i, j = 0, 0
        else:
            i = int(math.log(val, 1024)) + 1
            j = val / math.pow(1024, i)
            j, i = (j, i) if j > 0.5 else (j*1024, i-1)
        return ("{0:{1}f}"+text[i]).format(j, fmt[:-1])




# You can test it using:
#   wget https://partner-images.canonical.com/core/artful/current/ubuntu-artful-core-cloudimg-amd64-root.tar.gz
#   sudo bash -c "mkdir rootfs; tar xvf ubuntu-artful-core-cloudimg-amd64-root.tar.gz -C rootfs"
#   sudo python example.py bash

from __future__ import print_function
import subprocess
import os
import sys
from tmsyscall.unshare import unshare, CLONE_NEWNS, CLONE_NEWUTS, CLONE_NEWIPC, CLONE_NEWPID, CLONE_NEWNET
from tmsyscall.mount import mount, unmount, MS_BIND, MS_PRIVATE, MS_REC, MNT_DETACH
from tmsyscall.mount import mount_procfs
from tmsyscall.pivot_root import pivot_root
from os.path import exists


def setup_process_isolation():
    # Detach from parent's mount, hostname, ipc and net  namespaces
    unshare(CLONE_NEWNS| CLONE_NEWUTS | CLONE_NEWIPC| CLONE_NEWNET)

    # Set mount propagation to private recursively. Hopefully equivalent to
    #    mount --make-rprivate /
    # This is needed to prevent mounts in this container leaking to the parent.
    mount('none', '/', None, MS_REC|MS_PRIVATE, "")

    # The bind mount call is needed to satisfy a requirement of the `pivotroot` command
    # the OS requires that `pivotroot` be used to swap two filesystems that are not part of the same tree
    mount("rootfs", "rootfs", "", MS_BIND|MS_REC, "")
    if not exists("rootfs/.old_root"):
        os.makedirs("rootfs/.old_root", 0o700)
    pivot_root("rootfs", "rootfs/.old_root")
    os.chdir("/")

    # We don't want the host root to be available to the container
    unmount("/.old_root", MNT_DETACH)
    os.rmdir(".old_root")

    # Mount /proc for apps that need it
    if not exists("proc"):
        os.makedirs("proc", 0o700)
    mount_procfs('.')


def child():
    setup_process_isolation()
    proc = subprocess.Popen(sys.argv[1:], stdin=sys.stdin, stdout=sys.stdout, stderr=sys.stderr)
    proc.communicate()


def parent(child_pid):
    pid, status = os.waitpid(child_pid, 0)
    print("wait returned, pid = %d, status = %d" % (pid, status))


def main():
    # Detach from pid namespace so that our child get's a clean /proc with the new namespace
    unshare(CLONE_NEWPID)
    pid = os.fork()
    if pid == 0:
        child()
    else:
        parent(pid)

main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Install using distutils

Run:
    python setup.py install

to install this package.
"""
from setuptools import setup, find_packages
from os.path import join

with  open('requirements.txt') as r_file:
    requirements = r_file.read()

with  open('README.rst') as r_file:
    long_desc = r_file.read()

name = "pylibcontainer"

setup(
    version=open(join(name, 'version')).readline().strip("\r\n"),
    long_description=long_desc,
    install_requires=[x for x in requirements.splitlines() if x],
    packages=find_packages(),
    entry_points='''
        [console_scripts]
        pylibcontainer=pylibcontainer.__main__:pylibcontainer
        '''
)
